#!/bin/bash
# ************************************************************
# AltEPG Project Source Code Version Control
# $FileID: git::build/installTivo.bash $
# $Revision: altepg1.0d root 2012-03-03 00:50:19 +0000 final altepg1.0d $
# Description: AltEPG Installer
# ************************************************************

# This install script expects to find the software to be installed in the current directory
# It is driven off a file named "fileList.txt"
# The name of the package to be installed should be included in this file. If not included
# then the installation will be anonymous.
#
# fileList format:
# Active lines are of the form:
# <instruction><tab><data>
# Where <instruction> is a single Upper Case letter.  Lines that don't match this pattern are ignored.
# Instructions are:
# A - Add new file (data = pathname)
# M - Modify file (data = pathname)
# D - Delete file or directory (data = pathname)
# E - Execute file (data = pathname)
# H - Install hack (data = pathname)
# P - Package identifier (data = package name)
# R - Reboot required (data = restart message)
# V - Minimum version to which this installation can be applied (future functionality - ignored currently)
#
# All pathnames are relative to the current (installation) directory. The path to the location where the file is to be installed (or deleted)
# is generated from this pathname as follows:
#  - if the pathname begins "root/", then remove "root"
#  - prefix pathname with "/"
#    So  "root/etc/rc.d/rc.sysinit" will be installed as /etc/rc.d/rc.sysinit"
#    And "var/hack/bin/sendTivoMessage.tcl" will be installed as "/var/hack/bin/sendTivoMessage.tcl" 
# For "H", the pathname must be a .tgz file.  It will be unpacked into /var/hack
# For "R", the restart message is included in the TiVo message sent to the user. There is no automatic restart.

mode=$1
if [ -n "$mode" ]; then
  if [ "$mode" = "build" ]; then
    GIT_ROOT="../.."
    INSTALL_ROOT="."
  else
    echo "Unrecognised mode : $mode"
    exit 1
  fi
fi

SYSTEM=$(uname -r)
SYSNAME=$(uname -n)
if [ -z "${SYSTEM##*TiVo*}" ]; then
  SYS=TIVO
  LOGFILE="/var/log/install.log"
  LOCALLOGFILE="/var/tmp/install$$.log"
  MOUNT="mount"
  TIVO_ROOT=""
  HACKDIR=/var/hack
elif [ "$SYSNAME" = "Knoppix" ]; then
  SYS=BUILD
  LOGFILE="install.log"
  LOCALLOGFILE="install$$.log"
  MOUNT=":"
  TIVO_ROOT="/mnt/tivoRoot"
  HACKDIR=$TIVO_ROOT/hack
  mkdir -p "$TIVO_ROOT"
else
  SYS=OTHER
  LOGFILE="install.log"
  LOCALLOGFILE="install$$.log"
  MOUNT=":"
  TIVO_ROOT="/tmp/tivoRoot"
  HACKDIR=$TIVO_ROOT/hack
  mkdir -p "$TIVO_ROOT"
fi

echo -e "\nInstallation Log:\n" >"$LOCALLOGFILE"
echo "$(date +%Y-%m-%d:%H:%M:%S): Starting Installation" >>"$LOCALLOGFILE"
packageIdentifier="unspecified"
restartMessage=""

putlog()
{
  if [ -n "${1##DEBUG:*}" -o "$DEBUG" = 1 ]; then
    echo "$(date +%Y-%m-%d:%H:%M:%S): $1" >>"$LOGFILE"
    echo "$(date +%Y-%m-%d:%H:%M:%S): $1"
    if [ -n "${1##DEBUG:*}" ]; then
      echo "$1" >>"$LOCALLOGFILE"   # never send debug entries to local log, as this goes to tivo message
    fi
  fi
}

installFiles()
{
  if [ "$mode" = "build" ]; then
    while read line; do
      operation="${line%%	*}"
      if [ "$operation" != "D" ]; then
        file="${line##*	}"
        source="$GIT_ROOT/$file"
        target="$INSTALL_ROOT/$file"
        putlog "Copying '$source' to '$target'"
        basedir=$(dirname "$target")
        mkdir -p "$basedir"
        cp -v "$source" "$target"
      fi
    done
    return
  fi
  ecount=0
  $MOUNT -o remount,rw /
  displaceDir="$TIVO_ROOT/var/tmp/displacedFiles"
  mkdir -p "$displaceDir"
  while read line; do
    operation="${line%%	*}"
    file="${line##*	}"
    if [ -z "${line##*	*}" ]; then
      putlog "DEBUG: LINE=$line, op=$operation file=$file"
    fi
    partition="${file%%/*}"
    if [ "$partition" = "root" ]; then
      pathname="$TIVO_ROOT/${file#*/}"
    else
      pathname="$TIVO_ROOT/$partition/${file#*/}"
    fi
    if [ "$operation" = "A" -o "$operation" = "M" ]; then
      if [ "$operation" = "A" ]; then
        putlog "INFO: Installing new file $file to partition $partition as $pathname"
      else
        putlog "INFO: Installing new version of file $file to partition $partition as $pathname"
      fi
      if [ ! -f "$file" ]; then
        putlog "ERROR: $file not present in installation kit"
        error=1
      else
        if [ -f "$pathname" ]; then
	  if [ "$operation" = "A" ]; then
            putlog "WARNING: $pathname already exists, moving to $displaceDir"
	  else
	    putlog "INFO: $pathname found, moving old version to $displaceDir"
	  fi
          cp -p "$pathname" "$displaceDir"
          rm -f "$pathname"
	elif [ "$operation" = "M" ]; then
	  putlog "WARNING: $pathname not found in existing system"
	fi
	dir=${pathname%/*}
	putlog "DEBUG: target dir=$dir; copying $file to $pathname"
	if [ ! -d "$dir" ]; then
	  mkdir -p "$dir"
	fi
        cp -p "$file" "$pathname"
	if [ -f "$pathname" ]; then
	  putlog "INFO: $pathname installed"
	else
	  putlog "ERROR: $pathname not installed"
          error=1
	fi
      fi
    elif [ "$operation" = "D" ]; then
      putlog "INFO: Deleting $pathname"
      if [ -f "$pathname" ]; then
        putlog "INFO: $pathname found, moving old version to $displaceDir"
        cp -p "$pathname" "$displaceDir"
        rm -f "$pathname"
        if [ ! -f "$pathname" ]; then
          putlog "INFO: $pathname deleted"
        else
          putlog "ERROR: $pathname deletion failed"
          error=1
        fi
      elif [ -d "$pathname" ]; then
        putlog "INFO: $pathname found - is a directory"
        rm -f "$pathname"
        rm -f "$pathname"
        if [ ! -d "$pathname" ]; then
          putlog "INFO: $pathname deleted"
        else
          putlog "ERROR: $pathname deletion failed"
          error=1
        fi
      else
        putlog "WARNING: $pathname not found"
      fi
    elif [ "$operation" = "E" ]; then
      if [ "$SYS" = "TIVO" ]; then
        putlog "INFO: Executing $file"
        chmod 755 ./$file
        ./$file
        if [ $? != 0 ]; then
          putlog "ERROR: Failed to execute $file (error=$?)"
	  error=1
        fi
      else
        ecount=$[ecount+1]
	eprefix=$[ecount+100]
	filebase=$(basename "$file")
        base="$[ecount+100]_$filebase"
	phasenum=2
	pathname="$TIVO_ROOT/var/persist/$base.phase$phasenum"
        putlog "INFO: Copying $file to $pathname for execution on tivo boot"
	cp -p "$file" "$pathname"
	if [ -f "$pathname" ]; then
	  putlog "INFO: $pathname installed"
	else
	  putlog "ERROR: $pathname not installed"
          error=1
	fi
      fi
    elif [ "$operation" = "H" ]; then
      if [ ! -f "$file" ]; then
        putlog "ERROR: $file not present in installation kit"
        error=1
      elif [ -z "${file##*.tgz}" ]; then
        putlog "INFO: Installing package $file to $HACKDIR"
        gzip -df "$file"
	tarfile=${file%%.tgz}.tar
        here=$PWD
	cd "$HACKDIR"
	if [ "$SYS" = "BUILD" ]; then
	  tar xvf "$here/$tarfile"
	else
      	  cpio --unconditional --format=tar -i <"$here/$tarfile"
	fi
	cd "$here"
      else
        putlog "ERROR: Don't know how to install $file (file type isn't .tgz)"
	error=1
      fi
    elif [ "$operation" = "P" ]; then
      putlog "INFO: Package identifier : $file"
      packageIdentifier="$file"
    elif [ "$operation" = "V" ]; then
      currentVersion="$(cat /etc/altepg.version | head -1)"
      putlog "INFO: Current installed version is '$currentVersion'"
      putlog "INFO: This installation expects version $file or later to be installed already"
      putlog "INFO: but installation will proceed even if that is not the case"
    elif [ "$operation" = "R" ]; then
      putlog "INFO: Restart required"
      restartMessage="$file"
    elif [ -z "${operation##\#*}" ]; then
      : # ignore comment lines
    else
      putlog "ERROR: Unrecognised operation '$operation', ignored"
      error=1
    fi
  done
  $MOUNT -o remount,ro /
}

error=0
installFiles <fileList.txt

echo "$(date +%Y-%m-%d:%H:%M:%S): Finished Installation" >>"$LOCALLOGFILE"

# Send Tivo Message
if [ "$SYS" = "TIVO" ]; then
  if [ $error = 0 ]; then
    echo "New software ($packageIdentifier) has been installed on your TiVo. $restartMessage" \
    | cat - "$LOCALLOGFILE" \
    | /var/hack/bin/sendtivomesg.tcl "Tivo Installer" "New software installed" 
  else
    echo "An error occurred whilst installing new software ($packageIdentifier) on your TiVo" \
    | cat - "$LOCALLOGFILE" \
    | /var/hack/bin/sendtivomesg.tcl "Tivo Installer" "Software installation failed"
  fi
fi
rm "$LOCALLOGFILE"
exit $error
